﻿//*****************************************************************************
//* Copyright (c) 2005-2012 Miday Workshop All rights reserved.
//*
//* File: CalcGet.cpp
//*****************************************************************************

#ifdef _WINDOWS_PLAT
#include <windows.h>
#else  //_WINDOWS_PLAT
#include <gtk/gtk.h>
#include <unistd.h>
#include <time.h>
#include <stdint.h>
#endif  //_WINDOWS_PLAT
#include <stdio.h>
#include <wchar.h>
#include <string.h>
#include "CalcDef.h"
#include "SizeDef.h"
#include "MsgLocale.h"

#ifdef _WINDOWS_PLAT
extern SYSTEMTIME Just1, Just2;
#else  //_WINDOWS_PLAT
extern struct tm *Just1, *Just2;
#endif  //_WINDOWS_PLAT
int Tmin, Tsec;

//==============================================================================
//セル内の文字列データが数値かチェックする
//==============================================================================
static int Cell_num_chk ( UNICHAR *indata ) {
//    char *indata  入力　ＣＳＶデータ
//    int  num_flag 出力　数値フラグ　１そう　０違う
	int ia, end_p, num_flag;

	end_p   = MTRLEN(indata);
	num_flag = 1;

	//改行ありは左詰　数字かマイナス記号のみで右詰
	for ( ia=0; ia<end_p; ia++ ) {
#ifdef _UNICODE_TABLE
		if ( indata[ia] < (UNICHAR)'0' || indata[ia] > (UNICHAR)'9' ) {
			if ( indata[ia] != '-' ) {
#else  //_UNICODE_TABLE
		if ( indata[ia] < '0' || indata[ia] > '9' ) {
			if ( indata[ia] != '-' ) {
#endif  //_UNICODE_TABLE
				num_flag = 0;
				return num_flag;
			}
		}
	}
	return num_flag;
}

//==============================================================================
//セル内の行の文字列データの幅を計算する
//==============================================================================
static void Cell_info_line(int sa, int ea, int ja, int *charno, int *haba, int *takasa) {
//    int sa        入力　開始位置
//    int ea        入力　終了位置
//    int ja        入力　1バイトコードの文字数
//    int *haba     出力　幅(unicodeで、AsciiWide=1では * 2した値)
//    int *takasa   出力　高さ
//    int *charno   出力　セルに必要な文字数
	int na;
	*charno = *charno + ea - sa;
#ifdef _JPN
	na = ((ea - sa) * 2 - ja + 1) / 2;
#else  //_JPN
	na = (ea - sa) * 2 - ja;
#ifndef _UNICODE_TABLE
	na = na / 2;
#endif  //_UNICODE_TABLE
#endif  //_JPN
	if ( *haba != 0 ) {
		if ( *haba < na ) {
			*charno = *charno + (na - *haba) * (*takasa - 2);
			*haba = na;
		} else if ( *haba > na ) {
			*charno = *charno + (*haba - na);
		}
	} else {
		*haba = na;
	}
	return;
}

//==============================================================================
//セル内の文字列データの幅と高さを計算する
//==============================================================================
static void Cell_info_get ( UNICHAR *indata, int *haba, int *takasa, int *charno ) {
//    char *indata  入力　ＣＳＶデータ
//    int *haba     出力　幅(unicodeで、AsciiWide=1では * 2した値)
//    int *takasa   出力　高さ
//    int *charno   出力　セルに必要な文字数
	int ia, ja, ka;
	int end_p;

	end_p   = MTRLEN(indata) - 1;
	*haba   = 0;
	*takasa = 1;
	*charno = 0;
	ka = 0;
	for ( ia=0; ia<=end_p; ia++ ) {
		if ( indata[ia] == LIT_HLF ) {
			*takasa = *takasa + 1;
			ja = 0;
#ifdef _UNICODE_TABLE
#ifdef _JPN
			if ( AsciiWide == 1 ) {
#endif  //_JPN
				ja = HalfCheck(indata, ka, ia-1);
#ifdef _JPN
			}
#endif  //_JPN
#endif  //_UNICODE_TABLE
			Cell_info_line(ka, ia, ja, charno, haba, takasa);
			ka = ia + 1;
		}
	}
	ja = 0;
#ifdef _UNICODE_TABLE
#ifdef _JPN
	if ( AsciiWide == 1 ) {
#endif  //_JPN
		ja = HalfCheck(indata, ka, end_p);
#ifdef _JPN
	}
#endif  //_JPN
#endif  //_UNICODE_TABLE

	if ( end_p != ka - 1) {
		Cell_info_line( ka, end_p+1, ja, charno, haba, takasa);
	}

#ifndef _UNICODE_TABLE
		//シフトJISで日本語版で奇数バイトの場合は、+1
		if ( BYTELEN == 2 && *haba/2*2 != *haba ) {
			*haba = *haba + 1;
		}
#endif  //_UNICODE_TABLE
}

//==============================================================================
//Calcからセルの情報を取り込み編集する
//==============================================================================
int CalcGet() {
	int ia, iw, ih, ip, ix, irc;
	int iw2, ih2, rb1, rb2;
	int err_code;

//------------------------------------------------------------------------------
//Calcを起動し、セルの情報を取り込む
//------------------------------------------------------------------------------
	irc = CalcAcc();
	if ( irc == MEMORY_ERROR || irc == CANCEL_RETURN || irc == FINISHING_MESSAGE ) {//FINISHING_MESSAGEは下位でメッセージ表示済み
		return FINISHING_MESSAGE;
	}
	if ( irc != 0 ) {
		switch ( Proc_pos ) {
		case 0:
			break;
		case 1:
			irc = MessageDisp( MSG_ER_CAL1_CODE );
			break;
		case 2:
			irc = MessageDisp( MSG_ER_CAL2_CODE );
			break;
		case 4:
			irc = MessageDisp( MSG_ER_CLP9_CODE );
			break;
		default://case 3 case 5
			irc = MessageDisp( MSG_ER_CNV_CODE );
			break;
		}
		return FINISHING_MESSAGE;
	}

#ifdef _WINDOWS_PLAT
	Sleep(500);
#else  //_WINDOWS_PLAT
	sleep(1);
#endif  //_WINDOWS_PLAT
//------------------------------------------------------------------------------
//セルの情報を元に罫線の情報を作成する
//------------------------------------------------------------------------------
	for ( ih=0; ih<Row_no; ih++ ) {
		for ( iw=0; iw<Col_no; iw++ ) {

#if defined(_JPN) || defined(_ENG)
			//  結合セルの幅等をリセットする
			if ( hyo[ih*Col_no+iw].merge[AR_FLAG] != 0 && hyo[ih*Col_no+iw].merge[AR_FLAG] != 1 ) {
				MTRCPY(hyo[ih*Col_no+iw].text.data, hyo[ih*Col_no+iw].text.size+1, LIT_NULL_S);
				hyo[ih*Col_no+iw].takasa   = 0;
				hyo[ih*Col_no+iw].haba     = 0;
				hyo[ih*Col_no+iw].charno   = 0;
				hyo[ih*Col_no+iw].num_flag = 0;
			} else {

				if ( hyo[ih*Col_no+iw].text.size == -1 ) {//ダミーの領域を確保
					if ( CharLenNewAlloc(&hyo[ih*Col_no+iw].text, 10) == MEMORY_ERROR ) {
						return MEMORY_ERROR;
					}
					for ( ip=0; ip<4; ip++ ) {
						hyo[ih*Col_no+iw].line[ip] = 0;
					}
				}
				Cell_info_get ( hyo[ih*Col_no+iw].text.data, &hyo[ih*Col_no+iw].haba,
					&hyo[ih*Col_no+iw].takasa, &hyo[ih*Col_no+iw].charno );
				hyo[ih*Col_no+iw].num_flag = Cell_num_chk ( hyo[ih*Col_no+iw].text.data );
				if ( hyo[ih*Col_no+iw].merge[AR_FLAG] == 1 ) {
					for( iw2=hyo[ih*Col_no+iw].merge[AR_LEFT]; iw2<=hyo[ih*Col_no+iw].merge[AR_RIGHT]; iw2++ ) {
						hyo[ih*Col_no+iw2].takasa = hyo[ih*Col_no+iw].takasa;
					}
					for( ih2=hyo[ih*Col_no+iw].merge[AR_TOP]; ih2<=hyo[ih*Col_no+iw].merge[AR_BOTTOM]; ih2++ ) {
						hyo[ih2*Col_no+iw].haba = hyo[ih*Col_no+iw].haba;
					}
				}
				if ( hyo[ih*Col_no+iw].merge[AR_FLAG] != 1 ) {
					if ( hyo[ih*Col_no+iw].haba > yoko_p[iw].posw ) {
						yoko_p[iw].posw = hyo[ih*Col_no+iw].haba;
					}
					if ( hyo[ih*Col_no+iw].takasa > tate_p[ih].posw ) {
						tate_p[ih].posw = hyo[ih*Col_no+iw].takasa;
					}
				}
			}
#endif  //_JPN _ENG
		}
	}
//------------------------------------------------------------------------------
//セルの情報を元に作成した罫線の情報をチェックする
//------------------------------------------------------------------------------
	err_code = 0;
	for ( ih=0; ih<Row_no; ih++ ) {
		for ( iw=0; iw<Col_no; iw++ ) {
			for ( ip=0; ip<4; ip++ ) {
				if ( hyo[ih*Col_no+iw].line[ip] != 0 ) {
					err_code = 1;
					break;
				}
			}
		}
	}
	if ( err_code == 0 ) {//IN_CALCDATAで罫線がない場合　IN_TEXTDATAはここでは質問しない
		if ( text_flag == IN_CALCDATA && Allline_flag == 0 ) {
#ifdef _WINDOWS_PLAT
			if ( MessageDisp( MSG_ALL_LINE_CODE ) != IDOK ) {
#else  //_WINDOWS_PLAT
			if ( MessageDisp( MSG_ALL_LINE_CODE ) != GTK_RESPONSE_OK ) {
#endif  //_WINDOWS_PLAT
				irc = MessageDisp( MSG_CANCLE_CODE );
				FreeArea();
				return CANCEL_RETURN;
			} else {
				Allline_flag = 1;
			}
		}
		for ( ih=0; ih<Row_no; ih++ ) {
			for ( iw=0; iw<Col_no; iw++ ) {
				for ( ip=0; ip<4; ip++ ) {
					hyo[ih*Col_no+iw].line[ip] = 1;
				}
			}
		}
	}
//------------------------------------------------------------------------------
//結合セルの罫線の補正をする
//------------------------------------------------------------------------------
	for ( ih=0; ih<Row_no; ih++ ) {
		for ( iw=0; iw<Col_no; iw++ ) {
			if ( hyo[ih*Col_no+iw].merge[AR_FLAG] == 1 ) {
				if ( hyo[ih*Col_no+iw].merge[AR_RIGHT] >= Col_no || hyo[ih*Col_no+iw].merge[AR_BOTTOM] >= Row_no ) {
					//将来は、エリアの再確保にする
					irc = MessageDisp( MSG_ER_AREA_CODE );
					FreeArea();
					return FINISHING_MESSAGE;
				}
				rb1 = hyo[(ih)*Col_no+iw].line[AR_RIGHT];//Calcでは、左上のセルに右、下の罫線情報があるのでバックアップしておく
				rb2 = hyo[(ih)*Col_no+iw].line[AR_BOTTOM];//Calcでは、左上のセルに右、下の罫線情報があるのでバックアップしておく
				for ( iw2=hyo[ih*Col_no+iw].merge[AR_LEFT]; iw2<=hyo[ih*Col_no+iw].merge[AR_RIGHT]; iw2++ ) {
					for ( ih2=hyo[ih*Col_no+iw].merge[AR_TOP]; ih2<=hyo[ih*Col_no+iw].merge[AR_BOTTOM]; ih2++ ) {
						if ( iw2 != hyo[(ih)*Col_no+iw].merge[AR_LEFT] ) {
							hyo[(ih2)*Col_no+iw2].line[AR_LEFT] = 0;
						} else {
							hyo[(ih2)*Col_no+iw2].line[AR_LEFT] = hyo[(ih)*Col_no+iw].line[AR_LEFT];
						}
						if ( iw2 != hyo[(ih)*Col_no+iw].merge[AR_RIGHT] ) {
							hyo[(ih2)*Col_no+iw2].line[AR_RIGHT] = 0;
						} else {
							hyo[(ih2)*Col_no+iw2].line[AR_RIGHT] = rb1;
						}
						if ( ih2 != hyo[(ih)*Col_no+iw].merge[AR_TOP] ) {
							hyo[(ih2)*Col_no+iw2].line[AR_TOP] = 0;
						} else {
							hyo[(ih2)*Col_no+iw2].line[AR_TOP] = hyo[(ih)*Col_no+iw].line[AR_TOP];
						}
						if ( ih2 != hyo[(ih)*Col_no+iw].merge[AR_BOTTOM] ) {
							hyo[(ih2)*Col_no+iw2].line[AR_BOTTOM] = 0;
						} else {
							hyo[(ih2)*Col_no+iw2].line[AR_BOTTOM] = rb2;
						}
					}
				}
			}
		}
	}

	irc = MakeData();
	if ( irc == CANCEL_RETURN || irc == FINISHING_MESSAGE ) {//FINISHING_MESSAGEは下位でメッセージ表示済み
		return FINISHING_MESSAGE;
	}
	if ( irc != 0 ) {
		switch ( irc ) {
		case 3://エリアが矩形ではない
			break;
		case MEMORY_ERROR:
			irc = MessageDisp( MSG_ER_MEM2_CODE );
			break;
		default:
			irc = MessageDisp( MSG_ER_CNV_CODE );
			break;
		}
		return FINISHING_MESSAGE;
	}

//------------------------------------------------------------------------------
//クリップボード設定
//------------------------------------------------------------------------------
//MessageBox( hWnd, out_data.data,"output",0 );
#ifdef _LINUX_PLAT
	Window_Refresh();
#endif  //_LINUX_PLAT
	if ( Break_flag == 1 ) {
		irc = MessageDisp( MSG_CANCLE_CODE );
		FreeArea();
		Break_flag = 2;
		return CANCEL_RETURN;
	}
	ia = ClipWrite( out_data.data );
#ifdef _WINDOWS_PLAT
	Sleep(200);
#else  //_WINDOWS_PLAT
	sleep(1);
#endif  //_WINDOWS_PLAT
//------------------------------------------------------------------------------
//領域開放
//------------------------------------------------------------------------------
	FreeArea();

	ProgressView(100);

	//時間測定　処理時間が24時間を超えることは想定しない
	//超えた場合は、mod(x,24hr) or mod(x,48hr)
	if ( TimeLog == 1 ) {
#ifdef _WINDOWS_PLAT
		GetLocalTime(&Just2);
		ix = (Just2.wHour - Just1.wHour) * 60 + Just2.wMinute - Just1.wMinute;
		ix = ix * 60 + Just2.wSecond - Just1.wSecond;
		if ( Just2.wDay != Just1.wDay ) ix = ix + 24 * 60 * 60;
#else  //_WINDOWS_PLAT
		time_t aclock;
		int sh, sm, ss, sd;

		sh = Just1->tm_hour;
		sm = Just1->tm_min;
		ss = Just1->tm_sec;
		sd = Just1->tm_mday;

		time(&aclock);
		Just2 = localtime(&aclock);
		ix = (Just2->tm_hour - sh) * 60 + Just2->tm_min - sm;
		ix = ix * 60 + Just2->tm_sec - ss;
		if ( Just2->tm_mday != sd ) ix = ix + 24 * 60 * 60;
#endif  //_WINDOWS_PLAT
		Tmin = ix / 60;
		Tsec = ix - Tmin * 60;

		irc = MessageDisp( MSG_NOMAL_TIME_CODE );
	} else {
		irc = MessageDisp( MSG_NOMAL_CODE );
	}

	return NOMAL_RETURN;
}
